@@ -3,7 +3,7 @@ package Module::Refresh;
 use strict;
 use vars qw( $VERSION %CACHE );
 
-$VERSION = "0.04";
+$VERSION = "0.06";
 
 # Turn on the debugger's symbol source tracing
 BEGIN { $^P |= 0x10 };
@@ -41,7 +41,7 @@ Initialize the module refresher.
 sub new {
     my $proto = shift;
     my $self = ref($proto) || $proto;
-    $CACHE{$_} = $self->mtime($INC{$_}) for ( keys %INC );
+    $self->update_cache($_) for keys %INC;
     return ($self);
 };
 
@@ -50,6 +50,13 @@ sub new {
 Refresh all modules that have mtimes on disk newer than the newest ones we've got.
 Calls C<new> to initialize the cache if it had not yet been called.
 
+Specifically, it will renew any module that was loaded before the previous call
+to C<refresh> (or C<new>) and has changed on disk since then.  If a module was
+both loaded for the first time B<and> changed on disk between the previous call 
+and this one, it will B<not> be reloaded by this call (or any future one); you
+will need to update the modification time again (by using the Unix C<touch> command or
+making a change to it) in order for it to be reloaded.
+
 =cut
 
 sub refresh {
@@ -58,7 +65,9 @@ sub refresh {
     return $self->new if !%CACHE;
 
     foreach my $mod (sort keys %INC) {
-        if ( !$CACHE{$mod} or ( $self->mtime($INC{$mod}) ne $CACHE{$mod} ) ) {
+        if ( !$CACHE{$mod} ) {
+	    $self->update_cache($mod);
+	} elsif ( $self->mtime($INC{$mod}) ne $CACHE{$mod} ) {
             $self->refresh_module($mod);
         }
     }
@@ -82,7 +91,7 @@ sub refresh_module {
     local $@;
     eval { require $mod; 1 } or warn $@;
 
-    $CACHE{$mod} = $self->mtime( $INC{$mod} );
+    $self->update_cache($mod);
 
     return ($self);
 };
@@ -115,6 +124,19 @@ sub mtime {
     return join ' ', ( stat($_[1]) )[1, 7, 9];
 };
 
+=head2 update_cache $file
+
+Updates the cached "last modified" time for $file.
+
+=cut
+
+sub update_cache {
+    my $self = shift;
+    my $module_pm = shift;
+
+    $CACHE{$module_pm} = $self->mtime($INC{$module_pm});
+};
+
 =head2 unload_subs $file
 
 Wipe out subs defined in $file.
@@ -128,6 +150,7 @@ sub unload_subs {
     foreach my $sym (
         grep { index( $DB::sub{$_}, "$file:" ) == 0 } keys %DB::sub
     ) {
+        warn "Deleting $sym from $file" if ($sym =~ /freeze/);
         undef &$sym;
         delete $DB::sub{$sym};
     }
@@ -140,10 +163,12 @@ sub unload_subs {
 BEGIN {
     no strict 'refs';
     foreach my $sym (sort keys %{__PACKAGE__.'::'}) {
+        next if $sym eq 'VERSION'; # Skip the version sub, inherited from UNIVERSAL
         my $code = __PACKAGE__->can($sym) or next;
         delete ${__PACKAGE__.'::'}{$sym};
         *$sym = sub { goto &$code };
     }
+
 }
 
 1;
